#
# spec file
#
# Copyright (c) 2025 SUSE LLC
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#


%global flavor openmpi4-gnu-hpc%nil

%if "%{flavor}" == ""
%define pack_suff %{nil}
%else
%define pack_suff -%{flavor}

#
# Returns where MPI home is.
# Looks in the RPM based on the arg name for bin/mpicc
# and extracts mpihome from there.
# This handles mpicc being in the main RPM, in a -devel
# or in a dependency (when the top is an HPC meta RPM).
# However all the necessary packages MUST be installed.
%define hpc_mpi_home %( \
   check_package(){ (rpm -ql $1 | grep -E -q 'bin/mpicc$') && echo $1;};                \
   get_linked_pack(){ rpm -qR $1 | grep -v rpmlib | grep -v /sh | awk '{print $1}';}; \
   dirname $(dirname $(rpm -ql $(check_package %flavor || check_package %{flavor}-devel ||        \
   check_package $(get_linked_pack %flavor) || check_package $(get_linked_pack %{flavor}-devel)) | \
   grep -E 'bin/mpicc$')))

%endif

%define osu_ver  6.1
%define imb_ver  2021.3
%define imb_dir  mpi-benchmarks-IMB-v%{imb_ver}
%define osu_dir  osu-micro-benchmarks-%{osu_ver}

%define mpi_home %{hpc_mpi_home %flavor}
%define implem_list_dir %{_datadir}/mpitests/implem.d/
%define sles_pre_15 (0%{?sle_version} > 120000 && 0%{?sle_version} < 150000)
%define sles_pre_or_15 (0%{?sle_version} > 120000 && 0%{?sle_version} <= 150000)
%define sles_pre_16 (0%{?sle_version} > 120000 && 0%{?sle_version} < 160000)

Name:           mpitests%{pack_suff}
%if "%{flavor}" == ""
Summary:        MPI Benchmarks common files
License:        BSD-3-Clause AND CPL-1.0
Group:          Development/Languages/Other
%else
Summary:        MPI Benchmarks and tests for %{flavor}
License:        BSD-3-Clause AND CPL-1.0
Group:          Development/Languages/Other
%endif
Version:        3.2
Release:        16.2
URL:            http://www.openfabrics.org/downloads.htm
Source0:        http://mvapich.cse.ohio-state.edu/download/mvapich/osu-micro-benchmarks-%{osu_ver}.tar.gz
Source1:        https://github.com/intel/mpi-benchmarks/archive/IMB-v%{imb_ver}.tar.gz
Source3:        mpitests-runtests.sh
Source4:        mpitests-run.sh
Source100:      mpitests-rpmlintrc
Source101:      _multibuild
Patch1:         osu-fix-bad-return-values.patch
Patch2:         imb-cpp-flags.patch
BuildRoot:      %{_tmppath}/%{name}-%{version}-build
BuildRequires:  dos2unix
BuildRequires:  gcc-c++
# suse-hpc is not available for SLE < 15
%if 0%{?is_opensuse} || 0%{?sle_version} >= 150000
BuildRequires:  suse-hpc
%endif
%if "%{flavor}" != ""
BuildRequires:  %{flavor}
BuildRequires:  %{flavor}-devel
Requires:       %{flavor}
BuildRequires:  mpitests = %{version}
Requires:       mpitests = %{version}
%endif

%if "%{flavor}" == "mvapich2"
ExcludeArch:    %{arm}
%endif

%if  "%{flavor}" == "mvapich2-gnu-hpc"
%if %{sles_pre_15}
# Disable hpc builds for SLE12
ExclusiveArch:  do_not_build
%endif
ExcludeArch:    i586 %arm s390
%endif

%if "%{flavor}" == "mvapich2-psm2"
ExclusiveArch:  x86_64
%endif

%if "%{flavor}" == "mvapich2-psm2-gnu-hpc"
%if %{sles_pre_15}
# Disable hpc builds for SLE12
ExclusiveArch:  do_not_build
%else
ExclusiveArch:  x86_64
%endif
%endif

%if "%{flavor}" == "openmpi1-gnu-hpc"
%if %{sles_pre_15}
# Disable hpc builds for SLE12
ExclusiveArch:  do_not_build
%endif
%endif

%if "%{flavor}" == "openmpi4"
%if %{sles_pre_or_15}
# Disable openmpi4 builds for SLES up to (including) 15
ExclusiveArch:  do_not_build
%else
ExcludeArch:    ppc64
%endif
%endif

%if "%{flavor}" == "openmpi4-gnu-hpc"
%if %{sles_pre_or_15}
# Disable openmpi4 builds for SLES up to (including) 15
ExclusiveArch:  do_not_build
%else
ExcludeArch:    i586 %arm s390 ppc64
%endif
%endif

%if "%{flavor}" == "openmpi5"
%if %{sles_pre_16}
# Disable openmpi5 builds for SLES < 16
ExclusiveArch:  do_not_build
%else
ExcludeArch:    ppc64 %{arm} %ix86
%endif
%endif

%if "%{flavor}" == "openmpi5-gnu-hpc"
%if %{sles_pre_16}
# Disable openmpi5 builds for SLES < 16
ExclusiveArch:  do_not_build
%else
ExcludeArch:    %ix86 %arm s390 ppc64
%endif
%endif

# Disable mpich builds for SLE12 as it is not available
%if "%{flavor}" == "mpich-ofi" || "%{flavor}" == "mpich"
%if %{sles_pre_15}
ExclusiveArch:  do_not_build
%endif
%endif

# Disable mpich builds for SLE12 as it is not available
%if "%{flavor}" == "mpich-ofi-gnu-hpc" || "%{flavor}" == "mpich-gnu-hpc"
%if %{sles_pre_15}
ExclusiveArch:  do_not_build
%endif
ExcludeArch:    i586 %arm s390
%endif

%description
Set of popular MPI benchmarks: IMB v%{imb_ver} OSU benchmarks ver %{osu_ver}

%prep
%setup -c -q
%setup -T -D -a 1 -q
%patch -P 1 -p0
%patch -P 2 -p0

%if "%{flavor}" != ""
%build
echo echo %{mpi_home}
. %{mpi_home}/bin/mpivars.sh

export CFLAGS="%{optflags}"
export CXXFLAGS="%{optflags}"
export LDFLAGS="%{optflags}"

# IMB Build
make  CC=%{mpi_home}/bin/mpicc CXX=%{mpi_home}/bin/mpicxx \
     -C %{imb_dir}/ all

# OSU Build
( cd %{osu_dir} && \
  ./configure CC=%{mpi_home}/bin/mpicc CXX=%{mpi_home}/bin/mpicxx &&
  make all )

%install
# IMB
for imb_test in IMB-EXT IMB-IO IMB-MPI1 IMB-NBC IMB-RMA; do \
	install -D -m0755  %{imb_dir}/$imb_test %{buildroot}%{mpi_home}/tests/IMB/$imb_test;\
done
# OSU
make -C %{osu_dir} install prefix=%{buildroot}/usr libexecdir=%{buildroot}%{mpi_home}/tests

# Run script
sed -e s/@IMPLEM@/%{flavor}/g -e 's&@MPI_HOME@&%{mpi_home}&g' %{S:3}  > %{buildroot}%{mpi_home}/tests/runtests.sh
chmod 0755 %{buildroot}%{mpi_home}/tests/runtests.sh
mkdir -p %{buildroot}%{implem_list_dir}
echo %{mpi_home}/tests/runtests.sh > %{buildroot}%{implem_list_dir}/%{flavor}

%check
%bcond_with mpitests_check
%if %{with mpitests_check}
export SHORT=%{short}
export IMPLEM_LIST_DIR=%{implem_list_dir}
export BUILDROOT=%{buildroot}
%{_datadir}/mpitests/runtests.sh
%else
echo "Skipping tests"
%endif

%else

%build

%install
install -D -m0755 %{S:4} %{buildroot}%{_datadir}/%{name}/runtests.sh
mkdir -p %{buildroot}%{_datadir}/mpitests/implem.d/

%endif

%files
%defattr(-, root, root)
%dir %{_datadir}/mpitests/
%dir %{_datadir}/mpitests/implem.d/
%if "%{flavor}" == ""
%{_datadir}/%{name}/runtests.sh
%else
%doc %{imb_dir}/license/license.txt
%doc %{imb_dir}/license/use-of-trademark-license.txt
%doc %{osu_dir}/COPYRIGHT
%{mpi_home}/tests
%{_datadir}/mpitests/implem.d/%{flavor}
%endif

%changelog
* Wed Feb 19 2025 Nicolas Morey <nicolas.morey@suse.com>
- Add openmpi5 to flavours (jsc#PED-11356)
* Tue Jun 25 2024 Nicolas Morey <nicolas.morey@suse.com>
- Disable 32b builds of hpc flavours.
* Thu Feb 22 2024 pgajdos@suse.com
- Use %%patch -P N instead of deprecated %%patchN.
* Fri Oct 20 2023 Nicolas Morey <nicolas.morey@suse.com>
- Drop support for openmpi[123] (jsc#PED-7111)
- Prepare support for openmpi5
* Mon Aug  7 2023 Nicolas Morey <nicolas.morey@suse.com>
- Drop support for TrueScale (bsc#1212146)
* Tue Oct  4 2022 Nicolas Morey-Chaisemartin <nmoreychaisemartin@suse.com>
- Reenable LTO as it was not the real issue.
- Disable parallel build which is causing all kind of unexpected cbugs and crashes
* Thu Sep 29 2022 Nicolas Morey-Chaisemartin <nmoreychaisemartin@suse.com>
- Update OSU micro benchmark to 6.1
- Update Intel MPI Benchmarks to v2021.3
- Add osu-fix-bad-return-values.patch to fix compilation errors
- Add imb-cpp-flags.patch to fix IMB compilation flags
- Disable LTO as there are multiple failures with openMPI
* Mon Sep 12 2022 Dirk Müller <dmueller@suse.com>
- avoid using egrep, use grep -E to be compatible with newer grep versions
* Tue Jun  9 2020 Nicolas Morey-Chaisemartin <nmoreychaisemartin@suse.com>
- Update OSU micro benchmark to 5.6.3
- Update Intel MPI Benchmarks to v2019.6
* Thu Mar 19 2020 Nicolas Morey-Chaisemartin <nmoreychaisemartin@suse.com>
- Update OSU micro benchmark to 5.6.2
- Update Intel MPI Benchmarks to v2019.5
* Thu Mar 19 2020 Nicolas Morey-Chaisemartin <nmoreychaisemartin@suse.com>
- Add build for openmpi4
* Fri Jan 24 2020 Stefan Brüns <stefan.bruens@rwth-aachen.de>
- Fix syntax error in SLE version checks for openmpi3, sle_version
  is not defined for Factory.
* Mon Jan  6 2020 Nicolas Morey-Chaisemartin <nmoreychaisemartin@suse.com>
- Enable mpitests-openmpi3 for SLE >= 15-SP2
- Support releases with openmpi not yet renamed
* Wed Oct 16 2019 Nicolas Morey-Chaisemartin <nmoreychaisemartin@suse.com>
- openmpi was renamed into openmpi1
* Wed Aug 22 2018 nmoreychaisemartin@suse.com
- Fix compatibility for SLE12*
- Sync changelog with SLES12SP3
* Wed Mar 28 2018 nmoreychaisemartin@suse.com
- Add a bcond to force build against openmpi3 on SLE
* Mon Feb 12 2018 nmoreychaisemartin@suse.com
- Disable openmpi3 flavors in SLE
* Mon Jan 29 2018 nmoreychaisemartin@suse.com
- Update OSU micro benchmark to 3.4
  * New Features & Enhancements
  - Introduce new OpenSHMEM Non-blocking Benchmarks
  * osu_oshm_get_mr_nb
  * osu_oshm_get_nb
  * osu_oshm_put_mr_nb
  * osu_oshm_put_nb
  * osu_oshm_put_overlap
  - Automatically build OpenSHMEM 1.3 benchmarks when library support
    is detected
  - Add ability to specify min and max message size for point-to-point
    and one-sided benchmarks
  - Enhanced error handling for MPI benchmarks
  - Code clean-ups and unification of utility functions across benchmarks
  - Enhanced help messages and runtime parameters
  * Bug Fixes
  - Fix compile-time warnings
  - Fix peer calculation formula in UPC/UPC++ benchmarks
  - Fix correct number of warmup iterations in osu_barrier benchmark
* Fri Jan 26 2018 nmoreychaisemartin@suse.com
- Prepare for submission to Factory (bsc#1076593)
- Only run validation if mpitests_check bcond is not set
- Add missing copyright file for OSU
- Update IMB to IMB 2018.1 and switch source to github
- Remove obsolete norootforbuild
* Thu Jan  4 2018 nmoreychaisemartin@suse.com
- Disable mvapich2 tests on arm as they fail at the moment
* Fri Dec  8 2017 nmoreychaisemartin@suse.com
- Add --oversubscribe option for OpenMPI3 to fix fails on single
  CPU systems
* Thu Dec  7 2017 nmoreychaisemartin@suse.com
- Move hpc_mpi_home macro locally
* Mon Oct 30 2017 nmoreychaisemartin@suse.com
- Use new macro for mpi_home
- Remove the need for mpi-validate
* Mon Oct 30 2017 nmoreychaisemartin@suse.com
- Add multibuild mpich
* Mon Oct 23 2017 nmoreychaisemartin@suse.com
- Add gnu-hpc flavours
* Mon Sep 25 2017 nmoreychaisemartin@suse.com
- Add gen_multibuild.sh to autogenerate the _multibuild file
  from OBS project list
* Mon Sep 25 2017 nmoreychaisemartin@suse.com
- Support any install path for mpi implementation
- Remove forced psm2 btl for openmpi
* Wed Sep 13 2017 nmoreychaisemartin@suse.com
- Openmpi2 is not available on ppc64 anymore
* Mon Sep 11 2017 nmoreychaisemartin@suse.com
- Disable mvapich2-psm2 test when no PSM hardware is available
* Fri Sep  8 2017 nmoreychaisemartin@suse.com
- Dump all log files on error to ease debuggability
* Mon Jul 17 2017 nmoreychaisemartin@suse.com
- Add mpitests-run.sh as a wrapper to run the testsuite for
  all installed mpitests flavors
* Wed Jul 12 2017 nmoreychaisemartin@suse.com
- Remove legacy mpitests tarball (unmaintained by OFED)
- Drop associated patches:
  - mpitests-link_to_libm.patch
  - mpitests-order_of_evaluation.patch
  - mpitests-codecleanup.patch
  - mpitests-rpm_opt_flags.patch
- Update used test suites:
  * OSU micro benchmark 5.3.2
  * IMB 2017 Update 2
* Mon Jul 10 2017 nmoreychaisemartin@suse.com
- Add mpitests-runtests.sh top run and check results of all
  installed tests
* Tue Jun 27 2017 nmoreychaisemartin@suse.com
- Add _multibuild to build the various flavours
- Add a build for mvapich2-psm2, mpich and mpich-ofi
- Add mpitests-IMB-all-tests.patch to build IMB tests for MPI-2
  features
- Add dependency to the associated mpi package
* Fri Jun 23 2017 nmoreychaisemartin@suse.com
- PSM2 is only available on x86_64
* Wed Apr 26 2017 nmoreychaisemartin@suse.com
- Build against openmpi-1.10. bsc#1035326
* Wed Jul  9 2014 pth@suse.de
- Remove the bogus copy of mpitests-mvapich2.dif.
* Tue Jun  3 2014 pth@suse.de
- Update to OFED 3.12 final.
* Fri Mar 28 2014 pth@suse.de
- Rename mpitests-mvapich2.dif to mpitests-order_of_evaluation.patch.
- Package the license files for the Intel benchmarks.
- Add mpitests-rpm_opt_flags.patch to pass RPM_OPT_FLAGS to compiler.
- Add mpitests-link_to_libm.patch to link to libm.
- Convert all text files to unix format.
- Remove the files refering to mvapich.
- Add mpitests-codecleanup.patch to fix a cased of uninitialised
  var being used.
* Fri Jan 24 2014 pth@suse.de
- Update to OFED 3.12 daily.
* Mon Jan 13 2014 pth@suse.de
- New package from SLE
* Fri Sep  6 2013 pth@suse.de
- Enable build on s390(x).
* Thu Oct 28 2010 jjolly@novell.com
- Update to v3.2 from OFED 1.5.2
* Tue Oct 28 2008 jjolly@suse.de
- Updated to verion 3.1 from OFED 1.4 rc3
- Split off mpitests-mvapich and mpitests-mvapich2 to avoid build
  conflict
* Wed Jul 11 2007 hvogel@suse.de
- Initial Package, Version 2.0
